header {
package org.antlr.java.crossref;
import java.util.Stack;
}
/** Java 1.4 AST Recognizer Grammar
 *
 * Author: (see java.g preamble)
 *
 * This grammar is in the PUBLIC DOMAIN
 */
class JavaPreRefsTreeParser extends JavaTreeParser;

options {
	importVocab = Java;
    ASTLabelType = "org.antlr.java.crossref.LocationAST";
}

{
	static final boolean printDebug = true;
//    DebugStack scopeStack = new DebugStack();
    Stack scopeStack = new Stack();
    CodeProject project = null;
    MethodScope currentMethodScope = null;

    public void setProject(CodeProject project) {
        this.project = project;
    }

    protected String fileName = null;
    public void setFileName(String f) {
        fileName = f;
    }
}

compilationUnit
{
	String packageName = null;
}
    :
		( packageName=packageDefinition
		{
	        String fullName = new String(packageName + "." + fileName);
		    FileScope fs = project.getFileScope(fullName);
		    scopeStack.push(fs);
		}
		|	/* nothing */
		{
	        String fullName = new String("[default]." + fileName);
		    FileScope fs = project.getFileScope(fullName);
		    scopeStack.push(fs);
		}
		)
		(importDefinition)*
		(typeDefinition)*
		{scopeStack.pop();}
	;

packageDefinition  returns [String id = null]
	:	#( PACKAGE_DEF id=identifier )
	;

importDefinition
	:	#( IMPORT packageOrClassIdentifier )
	;

/** only called from importDefinition */
packageOrClassIdentifier
{
    String r = null;
    Scope scope = null;
}
	:	IDENT
        {
        FileScope currentScope = (FileScope)scopeStack.peek();
        ClassScope cs=project.getOrPredefClassScope(#IDENT.getText());
        currentScope.addImport(cs);
		//if(JavaPreRefsTreeParser.printDebug && CodeProject.printDebug)
        // System.out.println("JavaPreRefsTreeParser:packageOrClassIdentifier " + "import class "+scope);
        }
	|	#( DOT r=identifier
           (    STAR
                {
                FileScope currentScope = (FileScope)scopeStack.peek();
                scope = project.getPackageScope(r);
                currentScope.addImport(scope);
   				//if(JavaPreRefsTreeParser.printDebug && CodeProject.printDebug)
                // System.out.println("JavaPreRefsTreeParser:packageOrClassIdentifier " + "import "+scope);
                }
           |    IDENT
                {
                String id=r+"."+#IDENT.getText();
                FileScope currentScope = (FileScope)scopeStack.peek();
                ClassScope cs=project.getOrPredefClassScope(id);
                currentScope.addImport(cs);
				//if(JavaPreRefsTreeParser.printDebug && CodeProject.printDebug)
                // System.out.println("JavaPreRefsTreeParser:packageOrClassIdentifier " + "import class "+cs);
                }
           )
         )
	;

typeDefinition
{
    ClassScope cs = null;
}
:	#(  CLASS_DEF modifiers IDENT
            {	if(JavaPreRefsTreeParser.printDebug && CodeProject.printDebug)
            		System.out.println("JavaPreRefsTreeParser:typeDefinition " + "class "+#IDENT.getText());}
            {
            Scope currentScope = (Scope)scopeStack.peek();
            String pName = currentScope.getFullyQualifiedName();
            cs = project.getClassScope(pName+"."+#IDENT.getText());
            }
            extendsClause[cs] implementsClause[cs]
            {scopeStack.push(cs);}
            objBlock
            {scopeStack.pop();}
         )
	|	#(  INTERFACE_DEF modifiers IDENT
            {if(JavaPreRefsTreeParser.printDebug && CodeProject.printDebug)
            	System.out.println("JavaPreRefsTreeParser:typeDefinition " + "interface "+#IDENT.getText());}
            {
            Scope currentScope = (Scope)scopeStack.peek();
            String pName = currentScope.getFullyQualifiedName();
            cs = project.getClassScope(pName+"."+#IDENT.getText());
            }
            extendsClause[cs]
            {scopeStack.push(cs);}
            interfaceBlock
            {scopeStack.pop();}
         )
	;

extendsClause[ClassScope def]
{
    String id = null;
}
	:	#(  EXTENDS_CLAUSE
            (   id=identifier
                {
                Scope currentScope = (Scope)scopeStack.peek();
                ClassScope cs = currentScope.resolveClass(id);
                def.addSuperClass(cs);
                }
            )*
         )
         {
         if ( def!=null && def.getSuperClasses()==null ) {
            ClassScope o = project.getClassScope("java.lang.Object");
            def.addSuperClass(o);
         }
         }
	;

implementsClause[ClassScope def]
{
    String id = null;
}
	:	#(  IMPLEMENTS_CLAUSE
            (   id=identifier
                {
                Scope currentScope = (Scope)scopeStack.peek();
                ClassScope cs = currentScope.resolveClass(id);
                def.addImplementedInterface(cs);
                }
            )*
         )
	;


interfaceBlock
	:	#(	OBJBLOCK
			(	methodDecl
			|	variableDef
			)*
		)
	;

typeSpec returns [ClassScope t=null]
	:	#(TYPE t=typeSpecArray)
	;

typeSpecArray returns [ClassScope t=null]
	:	#( ARRAY_DECLARATOR t=typeSpecArray )
	|	t=type
	;

type returns [ClassScope t=null]
{
    String r=null;
}
    :	r=identifier
        {
        Scope currentScope = (Scope)scopeStack.peek();
        t = currentScope.resolveClass(r);
        }
	|	bit:builtInType
		{
		t = project.getOrPredefClassScope(bit.getText());
		}
	;

objBlock
	:	#(	OBJBLOCK
			(	ctorDef
			|	methodDef
			|	fieldDef
			|	typeDefinition
			|	#(STATIC_INIT slist)
			|	#(INSTANCE_INIT slist)
			)*
		)
	;

fieldDef
{
    ClassScope t = null;
}
	:	#(VARIABLE_DEF modifiers t=typeSpec fieldDeclarator[t] varInitializer)
	;

fieldDeclarator[ClassScope t]
	:	IDENT
        {
            // set types of all fields now that we can resolve refs
            ClassScope currentScope = (ClassScope)scopeStack.peek();
            FieldVariable f = currentScope.getField(#IDENT.getText());
            f.setType(t);
        }
	|	LBRACK fieldDeclarator[t]
	;

methodDef
{
    ClassScope retType = null; // either class name or we don't care
}
	:	#(METHOD_DEF modifiers retType=typeSpec methodHead[retType] (slist)?)
        {scopeStack.pop(); scopeStack.pop();}
	;

ctorDef
	:	#(CTOR_DEF modifiers methodHead[null] (slist)?)
        {scopeStack.pop(); scopeStack.pop();}
	;

methodDecl
{
    ClassScope retType = null; // either class name or we don't care
}
	:	#(METHOD_DEF modifiers retType=typeSpec methodHead[retType])
	    {scopeStack.pop(); scopeStack.pop();}
	;

methodHead[ClassScope retType]
{ StringBuffer signature = new StringBuffer();
  Scope currentScope = null;
  String temp;
}
	:	methH:IDENT
        {
        	//Make a method scope, don't tell the project about it, yet
        	//Want to wait for signature to make it unique
            currentScope = (Scope)scopeStack.peek();
            String pName = currentScope.getFullyQualifiedName();
            MethodScope ms = new MethodScope(project,methH.getText(),currentScope);
            ms.setSourceLocation(fileName, methH.getLine(), methH.getColumn());
//            MethodScope ms = project.getMethodScope(pName+"."+#IDENT.getText());
            scopeStack.push(ms);
            ParameterScope ps =
                    new ParameterScope(project,"[parameters]",ms);
            scopeStack.push(ps);
            ms.setReturnType(retType);
            ms.setParameterScope(ps);
            currentMethodScope = ms;
            //Setup for parameters
        	signature.append("(");
        }
        #( PARAMETERS 
        (temp=parameterDef
        { if( temp != null)
        	signature.append(Signature.typeToSignature(temp)); 
          else
        	signature.append(Signature.UNKNOWN); 
          }
        	
        )* 
        
        ) (throwsClause)?
        {
           	signature.append(")");
           	if(ms.getReturnType() != null)
           	{
           		temp = ms.getReturnType().getFullyQualifiedName();
           		signature.append(Signature.typeToSignature(temp));
           	}
           	ms.setSignature(signature.toString());
           	currentScope.addScopeMember(ms);
        }
   
        	
	;

slist
	:	#(  SLIST
            {
            // local {...} blocks are not kept in the MethodScope.
            // They are pushed on temporarily for resolving variables
            // but are not stored for use later. Revisit this in refs.g.
            Scope currentScope = (Scope)scopeStack.peek();
            CodeBlockScope cbs =
                new CodeBlockScope(project,"[slist]",currentScope);
            scopeStack.push(cbs);
            }
            (stat)*
            {Scope s = (Scope)scopeStack.pop(); 
           	if(JavaPreRefsTreeParser.printDebug && CodeProject.printDebug)
	           	{
               	System.err.println("JavaPreRefsTreeParser:slist") ;
				s.dump(); 
    	       	}
            }
         )
	;

variableDef
{
    ClassScope t = null;
}
	:	#(VARIABLE_DEF modifiers t=typeSpec variableDeclarator[t] varInitializer)
	;

variableDeclarator[ClassScope t]
	:	varDecl:IDENT
        {
            Scope currentScope = (Scope)scopeStack.peek();
            LocalVariable local =
                new LocalVariable(project,varDecl.getText(),currentScope);
            local.setSourceLocation(fileName, varDecl.getLine(), varDecl.getColumn());
            local.setType(t);
            currentScope.addScopeMember(local);
        }
	|	LBRACK variableDeclarator[t]
	;

parameterDef returns [String vartype=null]
{
    ClassScope t = null;
}
	:	#(PARAMETER_DEF modifiers t=typeSpec paramDef:IDENT )
        {
            Scope currentScope = (Scope)scopeStack.peek();
            LocalVariable local =
                new LocalVariable(project,paramDef.getText(),currentScope);
            local.setSourceLocation(fileName, paramDef.getLine(), paramDef.getColumn());
            local.setType(t);
            currentScope.addScopeMember(local);
            if(t != null)
	            vartype = t.getFullyQualifiedName();
        }
	;


identifier returns [String id=null]
{
    String r = null;
}
	:	IDENT {id=#IDENT.getText();}
	|	#( DOT r=identifier IDENT ) {id=r+"."+#IDENT.getText();}
	;
newExpression returns [ClassScope t=null]
	:	#(	"new" t=type
			(	newArrayDeclarator (arrayInitializer)?
			|	elist (objBlock)?
			)
		)
	;
	